from StringIO import StringIO
from cPickle import dumps

try:
    from hashlib import sha1 as sha
except ImportError:
    from sha import sha
    
import generation
import codegen
import clauses
import itertools
import tempfile
import codecs
import types
import utils
import etree
import marshal
import config
import copy
import sys
import imp
import os

GLOBALS = globals()

class Node(object):
    """Element translation class.

    This class implements the translation node for an element in the
    template document tree.

    It's used internally by the translation machinery.
    """

    symbols = config.SYMBOLS

    translate = None
    translation_name = None
    translation_domain = None
    translated_attributes = None
    skip = None
    cdata = None
    omit = None
    assign = None
    define = None
    macro = None
    use_macro = None
    extend_macro = None
    define_macro = None
    fill_slot = None
    define_slot = None
    condition = None
    repeat = None
    content = None
    include = None
    format = None
    dict_attributes = None
    dynamic_attributes = utils.emptydict()

    ns_omit = "http://xml.zope.org/namespaces/meta",
    
    def __init__(self, element):
        self.element = element

    @property
    def tag(self):
        tag = self.element.tag
        if '}' in tag:
            url, tag = tag[1:].split('}')
            for prefix, _url in self.element.nsmap.items():
                if prefix is not None and _url == url:
                    return "%s:%s" % (prefix, tag)
        return tag
    
    @property
    def text(self):
        if self.element.text is None:
            return ()
        if self.element.meta_structure is True:
            return (self.element.text,)
        else:
            return (utils.htmlescape(self.element.text),)

    @property
    def tail(self):
        if self.element.tail is None:
            return ()
        if self.element.meta_structure is True:
            return (self.element.tail,)
        else:
            return (utils.htmlescape(self.element.tail),)

    @property
    def ns_attributes(self):
        prefix_omit = set()
        ns_omit = list(self.ns_omit)

        root = self.element.getroottree().getroot()
        if root is None or utils.get_namespace(root) == config.META_NS:
            ns_omit.append("http://www.w3.org/1999/xhtml")

        prefix_omit = set()
        namespaces = self.element.nsmap.values()

        parent = self.element.getparent()
        while parent is not None:
            for prefix, ns in parent.nsmap.items():
                if ns in namespaces:
                    prefix_omit.add(prefix)
            parent = parent.getparent()

        attrs = dict(
            ((prefix and "xmlns:%s" % prefix or "xmlns", ns)
             for (prefix, ns) in self.element.nsmap.items()
             if ns not in ns_omit and prefix not in prefix_omit))

        return attrs

    @property
    def static_attributes(self):
        result = {}
        element_ns = self.element.nsmap.get(self.element.prefix)

        nsmap = self.element.nsmap.copy()
        nsmap.update(config.DEFAULT_NS_MAP)

        root = self.element.getroottree().getroot()
        omit_default_prefix = root.meta_omit_default_prefix

        for prefix, ns in nsmap.items():
            if ns not in self.ns_omit:
                attrs = utils.get_attributes_from_namespace(self.element, ns)
                if prefix is None or ns == element_ns:
                    attrs.update(
                        utils.get_attributes_from_namespace(self.element, None))

                for tag, value in attrs.items():
                    name = tag.split('}')[-1]

                    if prefix and (ns != element_ns or not omit_default_prefix):
                        result["%s:%s" % (prefix, name)] = value
                    elif omit_default_prefix:
                        result[name] = value

        if self.element.prefix is None:
            result.update(
                utils.get_attributes_from_namespace(self.element, None))

        return result

    @property
    def attribute_ordering(self):
        static = self.static_attributes
        ns = self.ns_attributes

        names = list(ns)

        # ensure that an "xmlns" attribute comes first
        names.sort(key=lambda name: name == 'xmlns', reverse=True)

        for key in self.element.attrib:
            name = utils.format_attribute(self.element, key)
            
            if name in static:
                names.append(name)

        return names
        
    @property
    def stream(self):
        return self.element.stream
    
    def update(self):
        self.element.update()
    
    def begin(self):
        self.stream.scope.append(set())
        self.stream.begin(self.serialize())
        
    def end(self):
        self.stream.end(self.serialize())
        self.stream.scope.pop()

    def body(self):
        if isinstance(self.skip, types.expression):
            assert isinstance(self.skip, types.value), \
                   "Dynamic skip condition can't be of type %s." % type(self.skip)
            condition = clauses.Condition(self.skip, invert=True)
            if len(self.element):
                condition.begin(self.stream)
        elif self.skip:
            return
        
        for element in self.element:
            element.node.update()

        for element in self.element:
            element.node.visit()

        if isinstance(self.skip, types.expression):
            if len(self.element):
                condition.end(self.stream)
                    
    def visit(self):
        assert self.stream is not None, "Must use ``start`` method."

        for element in self.element:
            if not isinstance(element, Element):
                self.wrap_literal(element)

        self.update()
        self.begin()
        self.body()
        self.end()

    def serialize(self):
        """Serialize element into clause-statements."""

        _ = []

        # i18n domain
        if self.translation_domain is not None:
            _.append(clauses.Define(
                self.symbols.domain, types.value(repr(self.translation_domain))))

        # variable definitions
        if self.define is not None:
            # as an optimization, we only define the `default`
            # symbol, if it's present in the definition clause (as
            # string representation)
            if self.symbols.default in repr(self.define):
                default = types.value(self.symbols.default_marker_symbol)
                _.append(clauses.Assign(default, self.symbols.default))
                
            for declaration, expression in self.define:
                _.append(clauses.Define(
                    declaration, expression, self.symbols.scope))

        # tag tail (deferred)
        tail = self.tail
        if self.fill_slot is None and self.translation_name is None:
            for part in reversed(tail):
                if isinstance(part, types.expression):
                    _.append(clauses.Write(part, defer=True))
                else:
                    _.append(clauses.Out(part, defer=True))
            
        # macro method
        macro = self.macro
        if macro is not None:
            _.append(clauses.Method(
                macro.name, macro.args, decorators=macro.decorators))
                
        # condition
        if self.condition is not None:
            _.append(clauses.Condition(self.condition))

        # repeat
        if self.repeat is not None:
            variables, expression = self.repeat
            if len(variables) > 1:
                repeat = clauses.Repeat(variables, expression, repeatdict=False)
            else:
                repeat = clauses.Repeat(variables, expression)
            _.append(repeat)

        # assign
        if self.assign is not None:
            for declaration, expression in self.assign:
                if len(declaration) != 1:
                    raise ValueError("Can only assign single variable.")
                variable = declaration[0]
                _.append(clauses.Assign(
                    expression, variable))        

        content = self.content
        omit = self.omit
        
        # macro slot definition; if the slot has been filled, a
        # specially named variable will be available in the local
        # scope (by internal convention).
        if self.define_slot:
            name = self.symbols.slot + utils.normalize_slot_name(self.define_slot)
            scope = set(itertools.chain(*self.stream.scope[1:]))
            if name in scope:
                exclude = set((name, self.symbols.scope)).union(
                    self.stream.scope[0])
                scope_args = tuple(
                    "%s=%s" % (variable, variable)
                    for variable in scope
                    if variable not in exclude)
                _.append(clauses.Condition(
                    types.value('isinstance(%s, basestring)' % name),
                    (clauses.Assign(
                    types.value('%s(%s)' % (
                    name, ", ".join((self.symbols.scope,) + scope_args))),
                    name),),
                    finalize=True,
                    invert=True))
                     
                content = types.value(name)
                omit = True
                
        # set dynamic content flag
        dynamic = content or self.translate is not None

        # if an attribute ordering is required, setting a default
        # trivial value for each attribute will ensure that the order
        # is preserved
        attributes = utils.odict()
        if self.attribute_ordering is not None:
            for name in self.attribute_ordering:
                attributes[name] = None

        # static attributes (including those with a namespace prefix)
        # are at the bottom of the food chain
        attributes.update(self.static_attributes)
        attributes.update(self.ns_attributes)

        # dynamic attributes
        dynamic_attrs = self.dynamic_attributes or ()
        dynamic_attr_names = []
        
        for variables, expression in dynamic_attrs:
            if len(variables) != 1:
                raise ValueError("Tuple definitions in assignment clause "
                                     "is not supported.")

            variable = variables[0]
            attributes[variable] = expression
            dynamic_attr_names.append(variable)

        # translated attributes
        translated_attributes = self.translated_attributes or ()
        for variable, msgid in translated_attributes:
            if msgid:
                if variable in dynamic_attr_names:
                    raise ValueError(
                        "Message id not allowed in conjunction with "
                        "a dynamic attribute.")

                value = types.value('"%s"' % msgid)

                if variable in attributes:
                    default = '"%s"' % attributes[variable]
                    expression = self.translate_expression(value, default=default)
                else:
                    expression = self.translate_expression(value)
            else:
                value = attributes.get(variable)
                if value is not None:
                    if variable not in dynamic_attr_names:
                        value = '"%s"' % value
                    expression = self.translate_expression(value)
                else:
                    raise ValueError("Must be either static or dynamic "
                                     "attribute when no message id "
                                     "is supplied.")

            attributes[variable] = expression

        # tag
        text = self.text
        if omit is not True:
            selfclosing = not text and not dynamic and len(self.element) == 0
            tag = clauses.Tag(
                self.tag, attributes,
                expression=self.dict_attributes, selfclosing=selfclosing,
                cdata=self.cdata is not None, defaults=self.static_attributes)
            if omit:
                _.append(clauses.Condition(
                    omit, [tag], finalize=False, invert=True))
            else:
                _.append(tag)

        # tag text (if we're not replacing tag body)
        if len(text) and not dynamic and not self.use_macro and not self.extend_macro:
            for part in text:
                if isinstance(part, types.expression):
                    _.append(clauses.Write(part))
                else:
                    _.append(clauses.Out(part))

        # dynamic content
        if content:
            msgid = self.translate
            if msgid is not None:
                if msgid:
                    raise ValueError(
                        "Can't use message id with dynamic content translation.")

                _.append(clauses.Assign(content, self.symbols.tmp))
                content = self.translate_expression(
                    types.value(self.symbols.tmp))
            else:
                value = types.value(repr(utils.serialize(self.element, omit=True)))
                _.insert(0, clauses.Assign(
                    value, "%s.value = %s" % (
                        self.symbols.default_marker_symbol, self.symbols.default)))
                
            _.append(clauses.Write(content))

        # dynamic text
        elif self.translate is not None and \
                 True in map(lambda part: isinstance(part, types.expression), text):
            if len(self.element):
                raise ValueError(
                    "Can't translate dynamic text block with elements in it.")

            init_stream = types.value('_init_stream()')
            init_stream.symbol_mapping['_init_stream'] = generation.initialize_stream
            
            subclauses = []
            subclauses.append(clauses.Define(
                types.declaration((self.symbols.out, self.symbols.write)), init_stream))

            for part in text:
                if isinstance(part, types.expression):
                    subclauses.append(clauses.Write(part))
                else:
                    part = part.strip().replace('  ', ' ').replace('\n', '')
                    if part != "":
                        subclauses.append(clauses.Out(part))

            # attempt translation
            subclauses.append(clauses.Assign(
                self.translate_expression(
                types.value('%(out)s.getvalue()'), 
                default=None), self.symbols.tmp))

            _.append(clauses.Group(subclauses))
            _.append(clauses.Write(types.value(self.symbols.tmp)))

        # include
        elif self.include:
            # compute macro function arguments and create argument string
            arguments = [
                "%s=%s" % (arg, arg) for arg in \
                set(itertools.chain(*self.stream.scope[1:]))]

            # XInclude's are similar to METAL macros, except the macro
            # is always defined as the entire template.

            # first we compute the filename expression and write it to
            # an internal variable
            _.append(clauses.Assign(self.include, self.symbols.include))

            # call template
            _.append(clauses.Write(
                types.template(
                "%%(xincludes)s.get(%%(include)s, %s).render_xinclude(%s)" % \
                (repr(self.format), ", ".join(arguments)))))
            
        # use macro
        elif self.use_macro or self.extend_macro:
            # for each fill-slot element, create a new output stream
            # and save value in a temporary variable
            kwargs = []
            scope = set(itertools.chain(*self.stream.scope[1:]))
            exclude = self.stream.scope[0]
            callbacks = set()
            undefined = set()

            if self.use_macro:
                # when using a macro, slots that are not used in this
                # template (by a slot definition), are marked as
                # undefined; note that a filled slot takes precedence
                # over an undefined slot---the logic is a little
                # convoluted, the reason being that macros essentially
                # behave like a black box
                defined = set()
                element = self.element
                while element is not None:
                    for define in etree.elements_with_attribute(
                        element, config.METAL_NS, 'define-slot'):
                        defined.add(define.node.define_slot)
                    element = element.getparent()

                for variable in scope:
                    if variable.startswith(self.symbols.slot):
                        name = variable[len(self.symbols.slot):]
                        if name not in defined:
                            undefined.add(variable)

            for element in etree.elements_with_attribute(
                self.element, config.METAL_NS, 'fill-slot'):
                if element.node.fill_slot is None:
                    # XXX this should not be necessary, but the above
                    # xpath expression finds non-"metal:fill-slot"
                    # nodes somehow on my system; this is perhaps due
                    # to a bug in the libxml2 version I'm using; we
                    # work around it by just skipping the element.
                    # (chrism)
                    continue

                callback = self.symbols.slot + \
                           utils.normalize_slot_name(element.node.fill_slot)
                callbacks.add(callback)
                
                remote_scope = self.symbols.scope+"_remote"
                scope_args = tuple(
                    "%s=%s" % (variable, variable)
                    for variable in scope
                    if variable not in exclude)

                init_stream = types.value('_init_stream()')
                init_stream.symbol_mapping[
                    '_init_stream'] = generation.initialize_stream

                subclauses = []
                subclauses.append(
                    clauses.Method(
                        callback, (remote_scope,) + scope_args + ('**_kw',),
                    types.value('%s.getvalue()' % self.symbols.out)))
                subclauses.append(
                    clauses.UpdateScope(self.symbols.scope, remote_scope))
                subclauses.append(
                    clauses.UpdateScope(self.symbols.scope, '_kw'))
                subclauses.append(clauses.Assign(
                    init_stream, (self.symbols.out, self.symbols.write)))
                subclauses.append(clauses.Visit(element.node))
                _.append(clauses.Group(subclauses))

            if self.use_macro:
                _.append(clauses.Assign(self.use_macro, self.symbols.metal))
            elif self.extend_macro:
                _.append(clauses.Assign(self.extend_macro, self.symbols.metal))

            arguments = {}
            for arg in scope:
                if arg not in exclude:
                    arguments[arg] = arg

            for key in undefined:
                self.stream.symbol_mapping[self.symbols.raise_exc] = utils.raise_exc
                raise_method_name = "raise%s" % key
                
                _.append(clauses.Assign(
                    types.value("lambda *args, **kwargs: %s(KeyError(%s))" % (
                        self.symbols.raise_exc, repr(key.lstrip(self.symbols.slot)))),
                    raise_method_name))
                
                arguments[key] = raise_method_name

            for callback in callbacks:
                arguments[callback] = callback

            # compute macro function arguments and create argument string
            if 'xmlns' in self.element.attrib:
                arguments['include_ns_attribute'] = repr(True)

            arguments = ", ".join("%s=%s" % kwarg for kwarg in arguments.items())
            value = types.value(
                "%s.render(%s)" % (self.symbols.metal, arguments))
            
            if self.use_macro:
                value.label = self.use_macro.label
            elif self.extend_macro:
                value.label = self.extend_macro.label
            _.append(clauses.Write(value))

        # translate body
        elif self.translate is not None:
            msgid = self.translate

            # subelements are either named or unnamed; if there are
            # unnamed elements, the message id must be dynamic
            named_elements = [e for e in self.element
                              if e.node.translation_name]
            
            unnamed_elements = [e for e in self.element
                                if not e.node.translation_name]

            if unnamed_elements and msgid:
                elements = ()
            else:
                elements = tuple(self.element)
            
            # if no message id is provided, there are two cases:
            #
            # 1) all dynamic content is assigned a translation name
            # 2) the message id needs to be generated dynamically
            #
            if not msgid and not unnamed_elements:
                msgid = self.create_msgid()

            if named_elements:
                mapping = self.symbols.mapping
                _.append(clauses.Assign(types.value('{}'), mapping))
            else:
                mapping = 'None'

            if not msgid:
                text = utils.htmlescape(self.element.text or "")
                _.append(clauses.Assign(types.value(repr(text)),
                                        self.symbols.msgid))

            # for each named block, create a new output stream
            # and use the value in the translation mapping dict
            for element in elements:
                init_stream = types.value('_init_stream()')
                init_stream.symbol_mapping[
                    '_init_stream'] = generation.initialize_stream

                subclauses = []
                subclauses.append(clauses.Define(
                    types.declaration((self.symbols.out, self.symbols.write)), init_stream))
                subclauses.append(clauses.Visit(element.node))

                # if the element is named, record it in the mapping
                if element in named_elements:
                    name = element.node.translation_name
                    
                    subclauses.append(clauses.Assign(
                        types.template('%(out)s.getvalue()'),
                        "%s['%s']" % (mapping, name)))

                    # when computing a dynamic message id, add a
                    # reference to the named block
                    if not msgid:
                        subclauses.append(clauses.Assign(
                            types.value(repr("${%s}" % name)), self.symbols.msgid))
                    
                # else add it to the dynamic message id
                else:
                    subclauses.append(clauses.Assign(
                        types.template('%(msgid)s + %(out)s.getvalue()'),
                        self.symbols.msgid))
                    
                _.append(clauses.Group(subclauses))

            if msgid:
                if elements:
                    value = types.value(repr(msgid))
                    default = self.symbols.marker
                else:
                    value = types.value(repr(msgid))
                    default = types.value(repr(
                        utils.serialize(self.element, omit=True)))
            else:
                default = value = types.template('%(msgid)s')
                
            _.append(clauses.Assign(
                self.translate_expression(
                    value, mapping=mapping, default=default), self.symbols.result))

            # write translation to output if successful, otherwise
            # fallback to default rendition; 
            result = types.value(self.symbols.result)
            result.symbol_mapping[self.symbols.marker] = generation.marker

            if msgid and elements:
                condition = types.template('%(result)s is not %(marker)s')
                _.append(clauses.Condition(
                    condition, [clauses.UnicodeWrite(result)], finalize=True))

                subclauses = []
                if self.element.text:
                    subclauses.append(clauses.Out(
                        utils.htmlescape(self.element.text)))
                for element in self.element:
                    name = element.node.translation_name
                    if name:
                        value = types.value("%s['%s']" % (mapping, name))
                        subclauses.append(clauses.Write(value))
                    else:
                        subclauses.append(clauses.Out(utils.serialize(element)))

                    for part in reversed(element.node.tail):
                        if isinstance(part, types.expression):
                            subclauses.append(clauses.Write(part))
                        else:
                            subclauses.append(clauses.Out(
                                utils.htmlescape(part)))

                if subclauses:
                    _.append(clauses.Else(subclauses))
            else:
                _.append(clauses.UnicodeWrite(result))
                
        return _

    def wrap_literal(self, element):
        index = self.element.index(element)

        t = self.element.makeelement(utils.meta_attr('literal'))
        t.meta_omit = ""
        t.tail = element.tail
        t.text = unicode(element)
        for child in element.getchildren():
            t.append(child)
        self.element.remove(element)
        self.element.insert(index, t)
        t.update()

    def create_msgid(self):
        """Create an i18n msgid from the tag contents."""

        out = StringIO()
        out.write(self.element.text)
        
        for element in self.element:
            name = element.node.translation_name
            if name:
                out.write("${%s}" % name)
                out.write(element.tail)
            else:
                out.write(element.tostring())

        msgid = out.getvalue().strip()
        msgid = msgid.replace('  ', ' ').replace('\n', '')

        return msgid

    def translate_expression(self, value, mapping=None, default=None):
        format = "%%(translate)s(%s, domain=%%(domain)s, mapping=%s, " \
                 "target_language=%%(language)s, default=%s)"
        template = types.template(
            format % (value, mapping, default))

        # add translate-method to symbol mapping
        translate = generation.fast_translate
        template.symbol_mapping[config.SYMBOLS.translate] = translate

        return template
    
class Element(etree.ElementBase):
    """Template element class.

    To start translation at this element, use the ``start`` method,
    providing a code stream object.
    """

    translator = None
    
    class node(Node):
        @property
        def omit(self):
            if self.element.meta_omit is not None:
                return self.element.meta_omit or True
            if self.element.meta_replace:
                return True

        @property
        def content(self):
            return self.element.meta_replace

    node = property(node)
    
    def start(self, stream):
        self._stream = stream
        self.node.visit()

    def update(self):
        pass
    
    @property
    def stream(self):
        while self is not None:
            try:
                return self._stream
            except AttributeError:
                self = self.getparent()

        raise ValueError("Can't locate stream object.")

    meta_cdata = etree.Annotation(
        utils.meta_attr('cdata'))
    
    meta_omit = etree.Annotation(
        utils.meta_attr('omit-tag'))

    meta_omit_default_prefix = etree.Annotation(
        utils.meta_attr('omit-default-prefix'))

    meta_structure = etree.Annotation(
        utils.meta_attr('structure'))

    meta_attributes = etree.Annotation(
        utils.meta_attr('attributes'))

    meta_replace = etree.Annotation(
        utils.meta_attr('replace'))

class MetaElement(Element):
    meta_cdata = etree.Annotation('cdata')
    
    meta_omit = True

    meta_structure = True
    
    meta_attributes = etree.Annotation('attributes')

    meta_replace = etree.Annotation('replace')

    meta_omit_default_prefix = etree.Annotation('omit-default-prefix')
    
class Compiler(object):
    """Template compiler. ``implicit_doctype`` will be used as the
    document type if the template does not define one
    itself. ``explicit_doctype`` may be used to explicitly set a
    doctype regardless of what the template defines."""

    doctype = None
    
    def __init__(self, tree, explicit_doctype=None,
                 xml_declaration=None, encoding=None, omit_default_prefix=True):
        self.xml_declaration = xml_declaration
        self.omit_default_prefix = omit_default_prefix
        self.tree = tree
        
        # explicit document type has priority over a parsed doctype
        if explicit_doctype is not None:
            self.doctype = explicit_doctype
        elif tree.docinfo.doctype:
            self.doctype = tree.docinfo.doctype
        
        if utils.coerces_gracefully(encoding):
            self.encoding = None
        else:
            self.encoding = encoding

        self.macros = self.tree.xpath(
            '//*[@metal:define-macro]/@metal:define-macro|'
            '//metal:*[@define-macro]/@define-macro',
            namespaces={'metal': config.METAL_NS})

    def __call__(self, macro=None, global_scope=True, parameters=()):
        root = copy.deepcopy(self.tree).getroot()

        if not isinstance(root, Element):
            raise ValueError(
                "Must define valid namespace for tag: '%s.'" % root.tag)

        # if macro is non-trivial, start compilation at the element
        # where the macro is defined
        if macro:
            elements = tuple(etree.elements_with_attribute(
                root, config.METAL_NS, 'define-macro', macro))

            if not elements:
                raise KeyError(macro)

            element = elements[0]
            element.meta_translator = root.meta_translator

            # if element is the document root, render as a normal
            # template, e.g. unset the `macro` mode
            if root is element:
                macro = None
            else:
                root = element

            # remove macro definition attribute from element
            if element.nsmap.get(element.prefix) == config.METAL_NS:
                del element.attrib['define-macro']
            else:
                del element.attrib[utils.metal_attr('define-macro')]

        # initialize code stream object
        stream = generation.CodeIO(
            root.node.symbols, encoding=self.encoding,
            indentation=0, indentation_string="\t")

        # transient symbols are added to the primary scope to exclude
        # them from being carried over when using macros
        for name, value in stream.symbols.as_dict().items():
            if value is config.TRANSIENT_SYMBOL:
                stream.scope[0].add(name)
        
        # initialize variable scope
        stream.scope.append(set(
            (stream.symbols.out,
             stream.symbols.write,
             stream.symbols.scope,
             stream.symbols.domain,
             stream.symbols.language) + \
            tuple(parameters)))

        # set up initialization code
        stream.symbol_mapping['_init_stream'] = generation.initialize_stream
        stream.symbol_mapping['_init_scope'] = generation.initialize_scope
        stream.symbol_mapping['_init_tal'] = generation.initialize_tal
        stream.symbol_mapping['_init_default'] = generation.initialize_default

        # add code-generation lookup globals
        stream.symbol_mapping.update(codegen.lookup_globals)

        if global_scope:
            assignments = (
                clauses.Assign(
                     types.value("_init_stream()"), ("%(out)s", "%(write)s")),
                clauses.Assign(
                     types.value("_init_tal()"), ("%(attributes)s", "%(repeat)s")),
                clauses.Assign(
                    types.value("_init_default()"), '%(default_marker_symbol)s'),
                clauses.Assign(
                     types.template("None"), "%(domain)s"))
        else:
            assignments = (
                clauses.Assign(
                    types.value("_init_tal()"), ("%(attributes)s", "%(repeat)s")),
                clauses.Assign(
                    types.value("_init_default()"), '%(default_marker_symbol)s'),
                clauses.Assign(
                     types.template("None"), "%(domain)s"))

        if stream.symbols.scope not in parameters:
            assignments += (
                clauses.Assign(
                    types.value("_init_scope()"), "%(scope)s"),
                )
            
        for clause in assignments:
            clause.begin(stream)
            clause.end(stream)

        if macro is not None:
            if macro == "" and 'xmlns' in root.attrib:
                del root.attrib['xmlns']

            wrapper = self.tree.parser.makeelement(
                utils.meta_attr('wrapper'), root.attrib)
            wrapper.append(root)
            root = wrapper
            
        # output XML headers, if applicable
        if not macro:
            header = ""
            if self.xml_declaration is not None:
                header += self.xml_declaration + '\n'
            if self.doctype:
                doctype = self.doctype + '\n'
                if self.encoding:
                    doctype = doctype.encode(self.encoding)
                header += doctype
            if header:
                out = clauses.Out(header)
                stream.scope.append(set())
                stream.begin([out])
                stream.end([out])
                stream.scope.pop()

        # add meta settings
        root.meta_omit_default_prefix = self.omit_default_prefix

        # start generation
        root.start(stream)
        body = stream.getvalue()

        # symbols dictionary
        __dict__ = stream.symbols.as_dict()

        # prepare args
        args = list(parameters)

        # prepare kwargs
        defaults = ["%s = None" % param for param in parameters]

        if not stream.symbols.language in args:
            args.append('%(language)s = None' % __dict__)

        # prepare globals
        _globals = ["from cPickle import loads as _loads"]
        for symbol, value in stream.symbol_mapping.items():
            _globals.append(
                "%s = _loads(%s)" % (symbol, repr(dumps(value))))
            
        # wrap generated Python-code in function definition
        if global_scope:
            source = generation.function_wrap(
                'render', args, _globals, body,
                "%(out)s.getvalue()" % __dict__)
        else:
            source = generation.function_wrap(
                'render', defaults, _globals, body)

        return ByteCodeTemplate(source)
        
class ByteCodeTemplate(object):
    """Template compiled to byte-code."""

    func = None

    def __init__(self, source):
        self.source = source
        
    def compile(self):
        suite = codegen.Suite(self.source)        
        _locals = {}

        if config.DEBUG_MODE:
            # write source code to temporary file on disk
            source = suite.source.encode('utf-8')
            try:
                f = open(self.tempfile, 'w')
            except AttributeError:
                fd, self.tempfile = tempfile.mkstemp()
                f = open(self.tempfile, 'w')
            f.write(codecs.BOM_UTF8+source)
            f.close()

            # load module using built-in source loader
            module_name = "chameleon_%s" % sha(source).hexdigest()
            module = imp.load_source(module_name, f.name, open(self.tempfile, 'r'))
            bind = module.bind
        else:
            _locals = {}
            exec suite.source in _locals
            bind = _locals["bind"]

        self.bind = bind
        self.func = bind()
        self.source = suite.source
        
    def __reduce__(self):
        reconstructor, (cls, base, state), kwargs = \
                       GhostedByteCodeTemplate(self).__reduce__()
        return reconstructor, (ByteCodeTemplate, base, state), kwargs

    def __del__(self):
        try:
            os.unlink(self.tempfile)
        except AttributeError:
            pass
        
    def __setstate__(self, state):
        self.__dict__.update(GhostedByteCodeTemplate.rebuild(state))

    @property
    def render(self):
        return self.func

class GhostedByteCodeTemplate(object):
    def __init__(self, template):
        self.code = marshal.dumps(template.bind.func_code)
        self.defaults = len(template.bind.func_defaults or ())
        self.source = template.source
        
    @classmethod
    def rebuild(cls, state):
        source = state['source']

        bind = sys.modules['types'].FunctionType(
            marshal.loads(state['code']), GLOBALS, "bind")

        func = bind()
        
        return dict(
            bind=bind,
            func=func,
            source=source)
            



import htmlentitydefs
import config
import utils
import base64
import re

import lxml.etree
import xml.parsers.expat

from cPickle import dumps, loads
from StringIO import StringIO

re_tag_start = re.compile(r'<(?!!)')

# this exception is imported for historic reasons
XMLSyntaxError = lxml.etree.XMLSyntaxError

def import_elementtree():
    """The ElementTree is used to validate output in debug-mode. We
    attempt to load the library from several locations."""
    
    try:
        import xml.etree.ElementTree as ET
    except:
        try:
            import elementtree.ElementTree as ET
        except ImportError:
            import cElementTree as ET
        
    return ET

def validate(string):
    """Wraps string as a proper HTML document and validates it by
    attempting to parse it using the active ElementTree parser."""
    
    validation_string = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" [ %s ]><div>%s</div>' % (
        utils.entities, string)
    
    try:
        import_elementtree().fromstring(validation_string)
    except xml.parsers.expat.ExpatError:
        raise ValidationError(string)

class ExpatParser(object):
    """XML tree parser using the ``xml.parsers.expat`` stream
    parser. This parser serve to accept template input which lacks a
    proper prefix namespace mapping or entity definitions. It also
    works around an issue where the expat parser incorrectly parses an
    element with trivial body text as self-closing."""
    
    root = None
    index = None
    
    # doctype
    doctype = None

    # xml-declaration
    xml_version = None
    encoding = None
    standalone = None
    parsing_external_entities = False
    
    def __init__(self, parser, body, expat, validator):
        self.parser = parser
        self.body = body
        self.expat = expat
        self._validate = validator
        
    def _get_element(self):
        try:
            return self._element
        except AttributeError:
            raise xml.parsers.expat.ExpatError(
                "Unable to parse document; no start-tag found.")

    def _set_element(self, element):
        self._element = element

    element = property(_get_element, _set_element)

    def _validate(self, element):
        pass
    
    def StartElementHandler(self, tag, attributes):
        attrs = utils.odict()
        while attributes:
            key = attributes.pop(0)
            value = attributes.pop(0)
            attrs[key] = value
        
        # update prefix to namespace mapping
        if self.root is None:
            self.index = self.expat.CurrentByteIndex
            nsmap = utils.odict()
        else:
            nsmap = self.root.nsmap.copy()

        # process namespace declarations
        for key, value in attrs.items():
            if key.startswith('xmlns:'):
                prefix, name = key.split(':')
                nsmap[name] = value
                del attrs[key]
            
        for key, value in attrs.items():
            try:
                prefix, name = key.split(':')
            except (ValueError, TypeError):
                continue

            del attrs[key]

            namespace = nsmap.get(prefix)
            if namespace is None:
                if self.root is not None:
                    element = self.element

                    while element is None:
                        namespace = element.nsmap.get(prefix)
                        if namespace is not None:
                            break
                        element = element.getparent()

                if namespace is None:
                    try:
                        namespace = config.DEFAULT_NS_MAP[prefix]
                    except KeyError:
                        raise KeyError(
                            "Attribute prefix unknown: '%s'." % prefix)

            attrs['{%s}%s' % (namespace, name)] = value

        # process tag
        try:
            prefix, name = tag.split(':')
            namespace = nsmap.get(prefix) or config.DEFAULT_NS_MAP[prefix]
            tag = '{%s}%s' % (namespace, name)
        except ValueError:
            pass

        # create element using parser
        element = self.parser.makeelement(tag, attrs, nsmap=nsmap)

        if self.root is None:
            document = []

            declaration = filter(
                lambda (key, value): value is not None,
                (('version', self.xml_version),
                 ('encoding', self.encoding),
                 ('standalone', self.standalone)))

            if declaration:
                document.append(
                    '<?xml %s?>' % "".join(
                        '%s="%s" ' % pair for pair in declaration))

            if self.doctype:
                doctype = []
                if (self.doctype["pubid"] is None and
                    self.doctype["sysid"] is None and
                    self.doctype["has_internal_subset"] and
                    self.doctype["entities"]):
                    doctype.append("<!DOCTYPE %(doctype_name)s [" % self.doctype)
                    for entity, repl in self.doctype["entities"]:
                        doctype.append('<!ENTITY %s "%s">' % (
                            entity,
                            repl.encode("ascii", "xmlcharrefreplace")))
                    doctype.append("]>")
                else:
                    doctype.append('<!DOCTYPE %(doctype_name)s PUBLIC "%(pubid)s" "%(sysid)s">' %
                                   self.doctype)
                self.doctype = "\n".join(doctype).encode("utf-8")
                document.append(self.doctype)

            # render element
            document.append(element.tostring())

            # parse document
            body = "\n".join(document).encode('utf-8')
            element = lxml.etree.fromstring(body, parser=self.parser)

            # set this element as tree root
            self.root = element
        else:
            self.element.append(element)

        # validate element
        self._validate(element)

        # set as current element
        self.element = element

    def EndElementHandler(self, name):
        if self.element.text is None and self.body[
            self.expat.CurrentByteIndex-2] != '/':
            self.element.text = ""
        self.element = self.element.getparent()

    def CharacterDataHandler(self, data):
        """Process ``data`` while comparing to input context to check
        for HTML entities (which must be preserved)."""

        
        context = self.body[self.expat.CurrentByteIndex:]
        text = u""

        while data:
            m = utils.re_entity.search(context)
            if m is None or m.start() >= len(data):
                text += data
                break

            n = utils.re_entity.match(data)
            if n is not None:
                length = n.end()
            else:
                length = 1

            text += context[0:m.end()]
            context = context[m.end():]
            data = data[m.start()+length:]
            
        if len(self.element) == 0:
            current = self.element.text or ""
            self.element.text = current + text
        else:
            current = self.element[-1].tail or ""
            self.element[-1].tail = current + text
            
    def ProcessingInstructionHandler(self, target, data):
        self.element.append(
            lxml.etree.PI(target, data))
        
    def StartCdataSectionHandler(self):
        element = self.parser.makeelement(
            utils.xhtml_attr('cdata'))
        element.meta_cdata = ""
        self.element.append(element)
        self.element = element            

    def EndCdataSectionHandler(self):
        self.element = self.element.getparent()

    def CommentHandler(self, text):
        if self.root is not None and self.element is not None:
            self.element.append(
                lxml.etree.Comment(text))
        
    def XmlDeclHandler(self, xml_version, encoding, standalone):
        self.xml_version = xml_version
        self.encoding = encoding

        if standalone:
            self.standalone = 'yes'
        else:
            self.standalone = 'no'
        
    def ExternalEntityRefHandler(self, context, base, sysid, pubid):
        self.parsing_external_entities = True
        try:
            parser = self.expat.ExternalEntityParserCreate(context)
            parser.ProcessingInstructionHandler = self.ProcessingInstructionHandler
            parser.ParseFile(StringIO(utils.entities))
            return 1
        finally:
            self.parsing_external_entities = False

    def DefaultHandler(self, userdata):
        if userdata.startswith('&'):
            return self.CharacterDataHandler(userdata)            
                
    def StartDoctypeDeclHandler(self, *args):
        doctype_name, sysid, pubid, has_internal_subset = args
        self.has_internal_subset = has_internal_subset
        self.doctype = {"doctype_name" : doctype_name,
                        "sysid": sysid,
                        "pubid": pubid,
                        "has_internal_subset": has_internal_subset,
                        "entities": []}

    def EntityDeclHandler(self, *args):
        if self.parsing_external_entities:
            return
        if self.doctype is not None and self.doctype["has_internal_subset"]:
            (entityName, is_parameter_entity, value,
             base, systemId, publicId, notationName) = args
            if value is not None:
                self.doctype["entities"].append((entityName, value))

class ValidationError(ValueError):
    def __str__(self):
        value, = self.args
        return "Insertion of %s is not allowed." % \
               repr(value)

class Parser(object):
    element_mapping = utils.emptydict()
    fallback = None
    
    def serialize(self, tree):
        return serialize(tree)

    def parse_xml(self, body):
        return self.parse(body)
    
    def parse_text(self, text):
        tree = self.parse('<meta xmlns="%s"></meta>' % config.META_NS)
        tree.getroot().text = text
        tree.getroot().meta_omit = ""
        return tree

    def parse(self, body):
        # set up namespace lookup class
        lookup = lxml.etree.ElementNamespaceClassLookup(
            fallback=lxml.etree.ElementDefaultClassLookup(self.fallback))
        for key, mapping in self.element_mapping.items():
            lookup.get_namespace(key).update(mapping)

        # set up lxml parser
        parser = lxml.etree.XMLParser(resolve_entities=False, strip_cdata=False)
        parser.setElementClassLookup(lookup)

        tree, self.doctype = parse(body, parser, self.validate)
        return tree

    def validate(self, element):
        pass

    # for backwards-compatibility, the parser object must be callable
    # which invokes the ``parse``-method
    def __call__(self, body):
        return self.parse(body)
    
class Annotation(property):
    def __init__(self, name, default=None):
        property.__init__(self, self._get, self._set)
        self.name = name
        self.default = default

    def _get(instance, element):
        value = element.attrib.get(instance.name)
        if value is not None:
            return loads(base64.decodestring(value))
        return instance.default

    def _set(instance, element, value):
        element.attrib[instance.name] = base64.encodestring(dumps(value))

def elements_with_attribute(element, ns, name, value=None):
    if value is None:
        expression = 'descendant-or-self::*[@prefix:%s] '\
                     '| descendant-or-self::prefix:*[@%s]' % (
            name, name)
    else:
        expression = 'descendant-or-self::*[@prefix:%s="%s"] '\
                     '| descendant-or-self::prefix:*[@%s="%s"]' % (
            name, value, name, value)

    return element.xpath(
        expression,
        namespaces={'prefix': ns})

class ElementBase(lxml.etree.ElementBase):
    def tostring(self):
        return lxml.etree.tostring(self)

def parse(body, parser, validator):
    """Parse XML document using expat and build lxml tree."""
    
    junk = ""
    tree = None
    parts = []

    offset = 0
    original_body = body
    
    while tree is None:
        # set up expat parser
        expat = xml.parsers.expat.ParserCreate()
        expat.ordered_attributes = 1
        expat.UseForeignDTD()
        expat.SetParamEntityParsing(
            xml.parsers.expat.XML_PARAM_ENTITY_PARSING_ALWAYS)

        # attach expat parser methods
        expatparser = ExpatParser(parser, body, expat, validator)
        for name, value in type(expatparser).__dict__.items():
            if isinstance(value, property):
                continue
            try:
                setattr(expat, name, getattr(expatparser, name))
            except AttributeError:
                pass

        try:
            # attempt to parse this body; if we're not successful,
            # this may be because the document source consists of
            # several 'parts'; although this is not valid XML, we do
            # support it, being a template engine, not a XML
            # validator :-)
            if body is not None:
                expat.Parse(body, 1)

            root = expatparser.root
            if parts:
                if root is not None:
                    parts.append(root)
                root = parser.makeelement(
                    utils.meta_attr('fragments'))
                for i, part in enumerate(parts):
                    if isinstance(part, basestring):
                        parts[i-1].tail = part
                    else:
                        root.append(part)
                tree = root.getroottree()
            elif root is not None:
                tree = root.getroottree()
            else:
                raise RuntimeError("Unexpected parsing error.")
            
        except xml.parsers.expat.ExpatError, e:
            offset += expat.CurrentByteIndex

            # if we are not able to find a tree root, we give up and
            # let the exception through; if the error status code is 3
            # (no element found) and we've already parsed some parts,
            # we ignore the error and stop parsing.
            code = getattr(e, 'code', -1)
            if code == 3:
                body = None
                
                if parts:
                    continue
                
                raise
            
            if expatparser.root is None:
                # the default exception won't provide correct
                # information about where in the document the
                # exception stems from; we can compute it manually
                # though.
                line = original_body[:offset].count('\n') + 1
                column = offset - original_body[:offset].rfind('\n')
                error_msg = str(e).split(':')[0]
                error_msg = "%s: line %d, column %d" % (
                    error_msg, line, column)
                raise xml.parsers.expat.ExpatError(error_msg)

            # add the root as a tree fragment and update the body
            # source to the next possible chunk
            parts.append(expatparser.root)
            body = body[:expatparser.index] + body[expat.CurrentByteIndex:]
            
            # a simple heuristic is used here to allow chunks of
            # 'junk' in-between the tree fragments
            m = re_tag_start.search(body)
            if m is not None:
                pos = m.start()
            else:
                pos = -1
            junk = body[:pos]
            body = body[pos:]
            parts.append(junk)
            offset += pos
            
    return tree, expatparser.doctype

def serialize(tree):
    """Serialize tree using lxml."""
    
    return lxml.etree.tostring(tree, encoding='utf-8')



import os
import sys
import config
import logging
import cPickle as pickle

logger = logging.getLogger("Chameleon")

class TemplateCache(object):
    def __init__(self, filename, version):
        self.filename = filename
        self.registry = {}
        self.version = version
        self.load()
        
    def __getitem__(self, key):
        return self.registry[key]

    def __setitem__(self, key, template):
        self.registry[key] = template
        self.save()

    def __len__(self):
        return len(self.registry)
    
    def get(self, key, default=None):
        return self.registry.get(key, default)
    
    @property
    def module_filename(self):
        return self.filename + os.extsep + config.CACHE_EXTENSION
    
    def load(self):
        try:
            module_filename = self.module_filename
            f = open(module_filename, 'rb')
        except IOError:
            return

        try:
            try:
                try:
                    version, version_info, registry = pickle.load(f)
                except EOFError:
                    pass
                
                if version != self.version:
                    raise ValueError("Version mismatch: %s != %s" % (
                        version, self.version))

                if sys.version_info != version_info:
                    raise ValueError("Python version mismatch: %s != %s" % (
                        version_info, self.version_info))

                self.registry.update(registry)
            except (ValueError, TypeError), e:
                logger.warn(
                    "Error loading cache for %s (%s)." % (self.filename, str(e)))
        finally:
            f.close()
        
    def save(self):
        try:
            f = open(self.module_filename, 'wb')
        except IOError:
            return

        try:
            data = self.version, sys.version_info, self.registry
            pickle.dump(data, f, protocol=2)
        finally:
            f.close()

    def clear(self):
        self.registry.clear()
                
    def purge(self):
        self.clear()
        self.save()



import translation
import template
import generation
import doctypes
import etree
import config
import utils
import types

from StringIO import StringIO
from cPickle import dumps, loads

class TestCompiler(translation.Compiler):
    def __call__(self, *args, **kwargs):
        template = translation.Compiler.__call__(self, *args, **kwargs)
        template.compile()
        template = loads(dumps(template))
        return template

def pyexp(*exps):
    return types.parts([types.value(exp) for exp in exps])

def setup_stream(encoding=None):
    class symbols(translation.Node.symbols):
        out = '_out'
        write = '_write'

    out = StringIO()
    write = out.write
    stream = generation.CodeIO(symbols, encoding=encoding)
    stream.scope.append(set())
    return out, write, stream

def render_xhtml(body, **kwargs):
    template = compile_template(mock_parser, mock_parser.parse, body)
    return template.render(**kwargs)    
    
def render_text(body, **kwargs):
    template = compile_template(mock_parser, mock_parser.parse_text, body)
    return template.render(**kwargs)    

def compile_xhtml(body):
    return compile_template(mock_parser, mock_parser.parse, body)

def compile_template(parser, parse_method, body, encoding=None,
                     macro=None, explicit_doctype=None, **kwargs):
    tree = parse_method(body)
    if '<?xml ' in body:
        xml_declaration = \
            """<?xml version="%s" encoding="%s" standalone="no" ?>""" % (
            tree.docinfo.xml_version, tree.docinfo.encoding)
    else:
        xml_declaration = None

    if not explicit_doctype is doctypes.no_doctype and not explicit_doctype:
        explicit_doctype = parser.doctype
    compiler = TestCompiler(
        tree, xml_declaration=xml_declaration,
        encoding=encoding, explicit_doctype=explicit_doctype)
    template = compiler(macro=macro, parameters=sorted(kwargs.keys()))    
    template.compile()
    
    return template

class MockElement(translation.Element):
    class Node(translation.Node):
        ns_omit = (
            "http://xml.zope.org/namespaces/meta",
            "http://www.w3.org/2001/XInclude")
        
        def __getattr__(self, name):
            return None

        @property
        def omit(self):
            if self.element.meta_omit is not None:
                return self.element.meta_omit or True
            if self.content:
                return True
            if self.include:
                return True

        @property
        def content(self):
            return self.element.meta_replace
        
        @property
        def cdata(self):
            return self.element.meta_cdata

        @property
        def include(self):
            href = self.element.xi_href
            if href is not None:
                return types.value(repr(href))

        @property
        def format(self):
            return self.element.xi_parse

    node = property(Node)

    xi_href = None
    xi_parse = None

class MockMetaElement(MockElement, translation.MetaElement):
    pass

class MockXiElement(MockElement):
    xi_href = utils.attribute('href')
    xi_parse = utils.attribute("parse", default="xml")

class MockParser(etree.Parser):
    element_mapping = {
        config.XHTML_NS: {None: MockElement},
        config.META_NS: {None: MockMetaElement},
        config.XI_NS: {None: MockXiElement}}

    fallback = MockElement
    
mock_parser = MockParser()

class MockMacros(template.Macros):
    def __getitem__(self, name):
        return self.get_macro(name)

class MockTemplate(object):
    def __init__(self, body, parser=mock_parser, doctype=None):
        self.body = body
        self.parser = parser
        self.doctype = doctype
        
    @property
    def macros(self):
        def render(name, parameters={}):
            template = compile_template(
                self.parser, self.parser.parse, self.body,
                macro=name, **parameters)
            return template.render(**parameters)
        return MockMacros(render)

    def render(self, **kwargs):
        template = compile_template(
            self.parser, self.parser.parse, self.body,
            explicit_doctype=self.doctype, **kwargs)
        return template.render(**kwargs)    

    __call__ = render



import errno
import os.path

def cache(func):
    def load(self, *args):
        template = self.registry.get(args)
        if template is None:
            self.registry[args] = template = func(self, *args)
        return template
    return load
    
class TemplateLoader(object):
    """Template loader tool. To load templates using relative
    filenames, pass a sequence of paths (or a single path) as
    ``search_path``; if ``auto_reload`` is set, templates will be
    reloaded when modified."""

    def __init__(self, search_path=None, auto_reload=False, parser=None):
        if search_path is None:
            search_path = []
        if isinstance(search_path, basestring):
            search_path = [search_path]
        self.search_path = search_path
        self.auto_reload = auto_reload
        self.parser = parser
        if parser is None:
            self.parser = getattr(self, 'default_parser', None)
        self.registry = {}

    @cache
    def load(self, filename, klass):
        if os.path.isabs(filename):
            return klass(filename, self.parser)

        for path in self.search_path:
            path = os.path.join(path, filename)
            try:
                return klass(
                    path, self.parser, auto_reload=self.auto_reload)
            except OSError, e:
                if e.errno != errno.ENOENT:
                    raise

        raise ValueError("Can not find template %s" % filename)




import re
import types

re_interpolation = re.compile(
    r'(?P<prefix>[^\\]\$|^\$)({((?P<expression>.*)})?|'
    '(?P<variable>[A-Za-z][A-Za-z0-9_]*))')

def interpolate(string, translator):
    """Splits up an interpolation string expression on the form
    ${<expression>} into a ``join`` expression.
    
    >>> def translator(string):
    ...     return types.value(string)

    >>> interpolate("${abc}", translator)
    (value('abc'),)

    >>> interpolate(" ${abc}", translator)
    (' ', value('abc'))

    >>> interpolate("abc${def}", translator)
    ('abc', value('def'))

    >>> interpolate("${def}abc", translator)
    (value('def'), 'abc')

    >>> interpolate("abc${def}ghi", translator)
    ('abc', value('def'), 'ghi')

    >>> interpolate("abc${def}ghi${jkl}", translator)
    ('abc', value('def'), 'ghi', value('jkl'))

    >>> interpolate("abc${def}", translator)
    ('abc', value('def'))

    >>> print interpolate(u"abc${ghi}", translator)
    (u'abc', value('ghi'))

    >>> print interpolate(u"}${abc}", translator)
    (u'}', value('abc'))
    
    """

    m = match_interpolate(string, translator)
    if m is None:
        return (string,)

    prefix = m.group('prefix')
    parts = []

    start = m.start() + len(prefix) - 1
    if start > 0:
        text = string[:start]
        parts.append(text)

    expression = m.group('expression')
    variable = m.group('variable')

    if expression:
        parts.append(translator(expression))
    elif variable:
        parts.append(translator(variable))

    rest = string[m.end():]
    if len(rest):
        parts.extend(interpolate(rest, translator))

    return tuple(parts)

def match_interpolate(string, translator):
    """Search for an interpolation and return a match.

    >>> def translator(string):
    ...     return types.value(string)

    >>> match_interpolate('${abc}', translator).group('expression')
    'abc'

    >>> match_interpolate(' ${abc}', translator).group('expression')
    'abc'

    >>> match_interpolate('abc${def}', translator).group('expression')
    'def'

    >>> match_interpolate('abc${def}ghi${jkl}', translator).group('expression')
    'def'

    >>> match_interpolate('$abc', translator).group('variable')
    'abc'

    >>> match_interpolate('${abc', translator)
    Traceback (most recent call last):
      ...
    SyntaxError: Interpolation expressions must be of the form ${<expression>} (${abc)

    """

    m = re_interpolation.search(string)
    if m is None:
        return None

    expression = m.group('expression')
    variable = m.group('variable')

    if expression:
        left = m.start()+len(m.group('prefix'))+1
        right = string.find('}')

        while right != -1:
            if right > left:
                match = string[left:right]
                try:
                    exp = translator(match)
                    break
                except SyntaxError:
                    pass
            right = string.find('}', right+1)
        else:
            raise

        string = string[:right+1]
        return re_interpolation.search(string)

    if m is None or (expression is None and variable is None):
        raise SyntaxError(
            "Interpolation expressions must be of the "
            "form ${<expression>} (%s)" % string)

    if expression and not m.group('expression'):
        raise SyntaxError(expression)

    return m



from compiler import ast, parse
from sourcecodegen import ModuleSourceCodeGenerator
from transformer import ASTTransformer
from chameleon.core import config

import __builtin__

CONSTANTS = frozenset(['False', 'True', 'None', 'NotImplemented', 'Ellipsis'])
SYMBOLS = config.SYMBOLS.as_dict().values()
UNDEFINED = object()

def flatten(list):
    """Flattens a potentially nested sequence into a flat list.
    """
    l = []
    for elt in list:
        t = type(elt)
        if t is set or t is tuple or t is list or t is frozenset:
            for elt2 in flatten(elt):
                l.append(elt2)
        else:
            l.append(elt)
    return l

def lookup_attr(obj, key):
    try:
        return getattr(obj, key)
    except AttributeError, e:
        try:
            return obj[key]
        except (KeyError, TypeError, AttributeError):
            raise e

def lookup_name(data, name):
    try:
        return data[name]
    except KeyError:
        raise NameError(name)
    
lookup_globals = {
    '_lookup_attr': lookup_attr,
    '_lookup_name': lookup_name,
    }       

class TemplateASTTransformer(ASTTransformer):
    """Concrete AST transformer that implements the AST transformations needed
    for code embedded in templates.

    """

    def __init__(self, globals):
        self.locals = [CONSTANTS]
        builtin = dir(__builtin__)
        self.locals.append(set(globals))
        self.locals.append(set(builtin))
        # self.names is an optimization for visitName (so we don't
        # need to flatten the locals every time it's called)
        self.names = set()
        self.names.update(CONSTANTS)
        self.names.update(builtin)
        self.names.update(globals)

    def visitConst(self, node):
        return node

    def visitAssName(self, node):
        if len(self.locals) > 1:
            if node.flags == 'OP_ASSIGN':
                self.locals[-1].add(node.name)
                self.names.add(node.name)
            else:
                self.locals[-1].remove(node.name)
                self.names.remove(node.name)
                return None
        return node

    def visitClass(self, node):
        if len(self.locals) > 1:
            self.locals[-1].add(node.name)
            self.names.add(node.name)
        self.locals.append(set())
        try:
            return ASTTransformer.visitClass(self, node)
        finally:
            self.locals.pop()

    def visitFor(self, node):
        self.locals.append(set())
        try:
            return ASTTransformer.visitFor(self, node)
        finally:
            self.locals.pop()

    def visitFunction(self, node):
        if len(self.locals) > 1:
            self.locals[-1].add(node.name)
            self.names.add(node.name)
        self.locals.append(set(node.argnames))
        self.names.update(node.argnames)

        try:
            return ASTTransformer.visitFunction(self, node)
        finally:
            self.locals.pop()

    def visitGenExpr(self, node):
        self.locals.append(set())
        try:
            return ASTTransformer.visitGenExpr(self, node)
        finally:
            self.locals.pop()

    def visitLambda(self, node):
        argnames = flatten(node.argnames)
        self.names.update(argnames)
        self.locals.append(set(argnames))
        try:
            return ASTTransformer.visitLambda(self, node)
        finally:
            self.locals.pop()

    def visitListComp(self, node):
        self.locals.append(set())
        try:
            return ASTTransformer.visitListComp(self, node)
        finally:
            self.locals.pop()

    def visitName(self, node):
        # If the name refers to a local inside a lambda, list comprehension, or
        # generator expression, leave it alone
        if not node.name in self.names and 'econtext' in self.names:
            # Otherwise, translate the name ref into a context lookup
            func_args = [ast.Name('econtext'), ast.Const(node.name)]
            node = ast.CallFunc(ast.Name('_lookup_name'), func_args)
        return node

    def visitGetattr(self, node):
        """Get attribute with fallback to dictionary lookup.

        Note: Variables starting with an underscore are exempt
        (reserved for internal use); as are the default system symbols.
        """
        
        if hasattr(node.expr, 'name') and node.expr.name.startswith('_') or \
               isinstance(node.expr, ast.Name) and node.expr.name in SYMBOLS:
            return ASTTransformer.visitGetattr(self, node)

        return ast.CallFunc(ast.Name('_lookup_attr'), [
            self.visit(node.expr), ast.Const(node.attrname)
            ])

class Suite(object):
    __slots__ = ['source', '_globals']

    mode = 'exec'
    
    def __init__(self, source, globals=()):
        """Create the code object from a string."""

        if isinstance(source, unicode):
            source = source.encode('utf-8')
            
        node = parse(source, self.mode)
        
        # build tree
        transform = TemplateASTTransformer(globals)
        tree = transform.visit(node)
        filename = tree.filename = '<script>'

        # generate source code
        self.source = ModuleSourceCodeGenerator(tree).getSourceCode()
        
    def __hash__(self):
        return hash(self.source)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.source)



from zope import interface

class ITALIterator(interface.Interface):
    """A TAL iterator

    Not to be confused with a Python iterator.
    """

    def next():
        """Advance to the next value in the iteration, if possible

        Return a true value if it was possible to advance and return
        a false value otherwise.
        """

class ITALESIterator(ITALIterator):
    """TAL Iterator provided by TALES

    Values of this iterator are assigned to items in the repeat namespace.

    For example, with a TAL statement like: tal:repeat="item items",
    an iterator will be assigned to "repeat/item".  The iterator
    provides a number of handy methods useful in writing TAL loops.

    The results are undefined of calling any of the methods except
    'length' before the first iteration.
    """

    def index():
        """Return the position (starting with "0") within the iteration
        """

    def number():
        """Return the position (starting with "1") within the iteration
        """

    def even():
        """Return whether the current position is even
        """

    def odd():
        """Return whether the current position is odd
        """

    def parity():
        """Return 'odd' or 'even' depending on the position's parity

        Useful for assigning CSS class names to table rows.
        """

    def start():
        """Return whether the current position is the first position
        """

    def end():
        """Return whether the current position is the last position
        """

    def letter():
        """Return the position (starting with "a") within the iteration
        """

    def Letter():
        """Return the position (starting with "A") within the iteration
        """

    def roman():
        """Return the position (starting with "i") within the iteration
        """

    def Roman():
        """Return the position (starting with "I") within the iteration
        """

    def item():
        """Return the item at the current position
        """

    def length():
        """Return the length of the sequence

        Note that this may fail if the TAL iterator was created on a Python
        iterator.
        """



import zope.testing
import unittest

OPTIONFLAGS = (zope.testing.doctest.ELLIPSIS |
               zope.testing.doctest.NORMALIZE_WHITESPACE)

import chameleon.core.config

def test_suite():
    filesuites = 'template.txt', 'codegen.txt', 'translation.txt', 'filecache.txt'
    testsuites = 'translation', 'clauses', 'parsing'

    chameleon.core.config.DISK_CACHE = False
    
    return unittest.TestSuite(
        [zope.testing.doctest.DocTestSuite(
        "chameleon.core."+doctest, optionflags=OPTIONFLAGS) \
         for doctest in testsuites] + 
        
        [zope.testing.doctest.DocFileSuite(
        doctest, optionflags=OPTIONFLAGS,
        package="chameleon.core") for doctest in filesuites]
        )

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from utils import emptydict

class expression:
    label = None
    symbol_mapping = emptydict()
    
class parts(tuple, expression):
    exceptions = (Exception,)
    
    def __new__(cls, value=()):
        if not isinstance(value, (tuple, list)):
            value = (value,)
        inst = tuple.__new__(cls, value)
        for part in value:
            if isinstance(part, expression):
                inst.symbol_mapping.update(part.symbol_mapping)
        return inst
    
    def __repr__(self):
        return 'parts'+tuple.__repr__(self)

class value(unicode, expression):
    def __init__(self, *args):
        super(value, self).__init__()
        self.symbol_mapping = {}
        
    def __repr__(self):
        try:
            r = repr(self.encode())
        except UnicodeEncodeError:
            r = unicode.__repr__(self)
            
        return "value(%s)" % r        

class template(value):
    def __repr__(self):
        return 'template(%s)' % value.__repr__(self)

class join(tuple, expression):
    def __repr__(self):
        return 'join'+tuple.__repr__(self)

class declaration(tuple):
    global_scope = False
    
    def __repr__(self):
        items = map(repr, self)
        if self.global_scope:
            items.append('global_scope=%s' % repr(self.global_scope))
        return 'declaration(%s)' % ', '.join(items)

class mapping(tuple):
    def __repr__(self):
        return 'mapping'+tuple.__repr__(self)

class definitions(tuple):
    def __repr__(self):
        return 'definitions'+tuple.__repr__(self)

class escape(parts):
    def __repr__(self):
        return 'escape'+tuple.__repr__(self)

class method(expression):
    def __init__(self, name, args, decorators=()):
        self.name = name
        self.args = args
        self.decorators = decorators
        self.symbol_mapping = {}
        
    def __repr__(self):
        return "%s%s(%s)" % (
            "".join("%s\n" % d for d in self.decorators),
            self.name, ", ".join(arg for arg in self.args))
        



from zope.i18n import interpolate
from zope.i18n import translate
from zope.i18nmessageid import Message

import utils
import types
import config

function_template = """\
def bind():
%(imports)s
\tdef %(name)s(%(arguments)s):
%(source)s
%(return)s
\treturn %(name)s
"""

def indent_block(text, level=2):
    return "\n".join(("\t" * level + s for s in text.split('\n')))
                     
def function_wrap(name, args, imports, source, return_expr=""):
    format_values = {
        'name': name,
        'imports': indent_block('\n'.join(imports), 1),
        'arguments': ', '.join(args),
        'source': indent_block(source),
        'return': indent_block("return %s" % return_expr)}
    
    return function_template % format_values

class Marker(object):
    def __nonzero__(self):
        return False

marker = Marker()

def fast_translate(msgid, domain=None, mapping=None, context=None,
                   target_language=None, default=None):
    """This translation function does not attempt to negotiate a
    language if ``None`` is passed."""

    if msgid is None:
        return
    
    if target_language is not None:
        return translate(
            msgid, domain=domain, mapping=mapping, context=context,
            target_language=target_language, default=default)
    
    if isinstance(msgid, Message):
        default = msgid.default
        mapping = msgid.mapping

    if default is None:
        default = unicode(msgid)

    if not isinstance(default, basestring):
        return default

    return interpolate(default, mapping)

def initialize_tal():
    return ({}, utils.repeatdict())

def initialize_default():
    return utils.default()

def initialize_stream():
    out = BufferIO()
    return (out, out.write)

initialize_scope = utils.econtext
    
class BufferIO(list):
    write = list.append

    def getvalue(self):
        return ''.join(self)
        
class CodeIO(BufferIO):
    """Stream buffer suitable for writing Python-code. This class is
    used internally by the compiler to manage variable scopes, source
    annotations and temporary variables."""

    t_prefix = '_tmp'
    v_prefix = '_tmpv'

    annotation = None
    
    def __init__(self, symbols=None, encoding=None,
                 indentation=0, indentation_string="\t"):
        BufferIO.__init__(self)
        self.symbols = symbols or object
        self.symbol_mapping = {}
        self.encoding = encoding
        self.indentation = indentation
        self.indentation_string = indentation_string
        self.queue = []
        self.scope = [set()]
        self.annotations = {}
        self._variables = {}
        self.t_counter = 0
        self.v_counter = 0
        self.l_counter = 0

    def new_var(self):
        self.v_counter += 1
        return "%s%d" % (self.v_prefix, self.v_counter)
        
    def save(self):
        self.t_counter += 1
        return "%s%d" % (self.t_prefix, self.t_counter)

    def restore(self):
        var = "%s%d" % (self.t_prefix, self.t_counter)
        self.t_counter -= 1
        return var

    def indent(self, amount=1):
        if amount > 0:
            self.flush()
            self.indentation += amount

    def outdent(self, amount=1):
        if amount > 0:
            self.flush()
            self.indentation -= amount

    def annotate(self, annotation):
        if annotation.label is not None:
            annotation = annotation.label

        # make sure the annotation is a base string type
        if isinstance(annotation, unicode):
            annotation = unicode(annotation)
        else:
            annotation = str(annotation)

        # encode unicode string if required
        if isinstance(annotation, unicode) and self.encoding:
            annotation = annotation.encode(self.encoding)
            
        self.annotation = self.annotations[self.l_counter] = annotation

    def out(self, value):
        if isinstance(value, types.expression):
            assert isinstance(value, types.value)
        elif isinstance(value, unicode) and self.encoding:
            value = value.encode(self.encoding, 'xmlcharrefreplace')
        self.queue.append(value)
            
    def flush(self):
        if self.queue:
            # optimize queue to merge adjacent strings
            optimized = []

            current = None
            while self.queue:
                next = self.queue.pop(0)

                if type(current) in (str, unicode) and \
                   type(next) in (str, unicode):
                    current = current + next
                    continue
                elif current is not None:
                    optimized.append(current)

                current = next
            else:
                optimized.append(current)            

            assert len(self.queue) == 0
            
            self.write(
                "%s(%s)" %
                (self.symbols.write, " + ".join(
                isinstance(expression, types.value) and expression or \
                repr(expression) for expression in optimized)))

    def write(self, string):
        if isinstance(string, unicode) and self.encoding:
            string = string.encode(self.encoding)

        self.l_counter += len(string.split('\n'))-1
        self.flush()
        
        indent = self.indentation_string * self.indentation

        # if a source code annotation is set, write it as a
        # triple-quoted string prior to the source line
        if self.annotation:
            if isinstance(self.annotation, unicode) and self.encoding:
                self.annotation = self.annotation.encode(self.encoding)
            BufferIO.write(
                self, "%s%s\n" % (indent, repr(self.annotation)))
            self.annotation = None
            
        BufferIO.write(self, indent + string + '\n')

    def getvalue(self):
        self.flush()
        return BufferIO.getvalue(self)

    def escape(self, variable, escape_double_quote=False):
        self.symbol_mapping[config.SYMBOLS.re_amp] = utils.re_amp
        self.write("if '&' in %s:" % variable)
        self.indent()
        self.write("if ';' in %s: %s = %s.sub('&amp;', %s)" % (
            variable, variable, config.SYMBOLS.re_amp, variable))
        self.write("else: %s = %s.replace('&', '&amp;')" % (variable, variable))
        self.outdent()
        self.write("if '<' in %s:" % variable)
        self.indent()
        self.write("%s = %s.replace('<', '&lt;')" % (variable, variable))
        self.outdent()
        self.write("if '>' in %s:" % variable)
        self.indent()
        self.write("%s = %s.replace('>', '&gt;')" % (variable, variable))
        self.outdent()
        if escape_double_quote:
            self.write("if '\"' in %s:" % variable)
            self.indent()
            self.write("%s = %s.replace('\"', '&quot;')" % (variable, variable))
            self.outdent()

    def convert_to_string(self, variable):
        self.write("if isinstance(%s, unicode):" % variable)
        self.indent()
        self.write("%s = %s.encode('%s', 'xmlcharrefreplace')" % (
            variable, variable, self.encoding))
        self.outdent()

    def coerce_to_string(self, variable):
        self.write("if isinstance(%s, unicode):" % variable)
        self.indent()
        self.write("%s = %s.encode('%s', 'xmlcharrefreplace')" % (
            variable, variable, self.encoding))
        self.outdent()
        self.write("elif not isinstance(%s, str):" % variable)
        self.indent()
        self.write("%s = str(%s)" % (variable, variable))
        self.outdent()

    def coerce_to_unicode(self, variable):
        """Coerces variable to unicode (actually ``str``, because it's
        much faster)."""
        
        self.write("if not isinstance(%s, basestring):" % variable)
        self.indent()
        self.write("%s = str(%s)" % (variable, variable))
        self.outdent()
        
    def begin(self, clauses):
        if isinstance(clauses, (list, tuple)):
            for clause in clauses:
                self.begin(clause)
        else:
            clauses.begin(self)
                
    def end(self, clauses):
        if isinstance(clauses, (list, tuple)):
            for clause in reversed(clauses):
                self.end(clause)
        else:
            clauses.end(self)



from zope import interface

import cgi
import sys
import interfaces
import htmlentitydefs
import re

types = sys.modules['types']

from UserDict import UserDict

# check if we're able to coerce unicode to str
try:
    str(u'La Pe\xf1a')
    unicode_required_flag = False
except UnicodeEncodeError:
    unicode_required_flag = True

def coerces_gracefully(encoding):
    if encoding != sys.getdefaultencoding() and unicode_required_flag:
        return False
    return True

entities = "".join((
    '<!ENTITY %s "&#%s;">' % (name, text) for (name, text) in \
    htmlentitydefs.name2codepoint.items()))

re_annotation = re.compile(r'^\s*u?[\'"](.*)[\'"]$')
re_entity = re.compile(r'&([A-Za-z]+|#[0-9]+);')
re_numeric_entity = re.compile(r'&#([0-9]+);')
re_amp = re.compile(r'&(?!([A-Za-z]+|#[0-9]+);)');
re_meta = re.compile(
    r'\s*<meta\s+http-equiv=["\']?Content-Type["\']?'
    r'\s+content=["\']?([^;]+);\s*charset=([^"\']+)["\']?\s*/?\s*>\s*',
    re.IGNORECASE)

XML_PREFIXES = [
    "<?xml",                      # ascii, utf-8
    "\xef\xbb\xbf<?xml",          # utf-8 w/ byte order mark
    "\0<\0?\0x\0m\0l",            # utf-16 big endian
    "<\0?\0x\0m\0l\0",            # utf-16 little endian
    "\xfe\xff\0<\0?\0x\0m\0l",    # utf-16 big endian w/ byte order mark
    "\xff\xfe<\0?\0x\0m\0l\0",    # utf-16 little endian w/ byte order mark
    ]

XML_PREFIX_MAX_LENGTH = max(map(len, XML_PREFIXES))

s_counter = 0
marker = object()

def sniff_type(text):
    """Return 'text/xml' if text appears to be XML, otherwise return None."""
    for prefix in XML_PREFIXES:
        if text.startswith(prefix):
            return "text/xml"
    return None

def handler(key=None):
    def decorate(f):
        def g(node):
            if key is None:
                return f(node, None)
            return f(node, node.get(key))
        g.__ns__ = key
        return g
    return decorate

class attribute(property):
    def __init__(self, tokens, factory=None, default=None, encoding=None):
        self.tokens = tokens
        self.factory = factory
        self.default = default
        self.encoding = encoding
        property.__init__(self, self.__call__)
        
    def __call__(attribute, element):
        value = attribute.get(element)
        if value is not None:
            encoding = attribute.encoding or element.stream.encoding
            if encoding:
                value = value.encode(encoding, 'xmlcharrefreplace')
            if attribute.factory is None:
                return value
            f = attribute.factory(element.translator)
            return f(value)

        if attribute.default is not None:
            return attribute.default

    def get(attribute, element):
        tokens = attribute.tokens
        for token in isinstance(tokens, tuple) and tokens or (tokens,):
            value = element.attrib.get(token)
            if value is not None:
                return value

def escape(string, quote=None, encoding=None):
    if not isinstance(string, unicode) and encoding:
        string = string.decode(encoding)
    else:
        encoding = None

    string = re_amp.sub('&amp;', string).\
             replace('<', '&lt;').\
             replace('>', '&gt;')

    if quote is not None:
        string = string.replace(quote, '\\'+quote)

    # force html entities for any non-ascii character
    def substitute(m):
        name = m.group(1)
        code = int(name)
        return '&%s;' % htmlentitydefs.codepoint2name[code]
            
    string = string.encode('ascii', 'xmlcharrefreplace').decode('ascii')
    string = re_numeric_entity.sub(substitute, string)

    if encoding:
        string = string.encode(encoding)

    return string

def htmlescape(text):
    while text:
        m = re_amp.search(text)
        if m is None:
            break
        text = text[:m.start()] + re_amp.sub("&amp;", text[m.start():], 1)
        
    return text.replace('<', '&lt;').replace('>', '&gt;')

re_normalize = re.compile(r'(^[0-9]|\b[^A-Za-z])')
def normalize_slot_name(name):
    return re_normalize.sub('_', name)
    
def serialize(element, encoding=None, omit=False):
    return "".join(serialize_element(element, encoding, omit=omit))

def serialize_element(element, encoding, omit):
    try:
        name = element.tag.split('}')[-1]
    except AttributeError:
        yield htmlescape(element.text); return

    if omit is False:
        # tag opening
        yield "<%s" % name

        # attributes
        for key, value in element.attrib.items():
            yield ' %s="%s"' % (key, escape('"', encoding=encoding))

        # elements with no text which have no children are self-closing.
        if element.text is None and len(element) == 0:
            yield ' />'; return

        yield '>'
            
    if element.text is not None:
        yield htmlescape(element.text)

    for child in element:
        for string in serialize_element(child, encoding, False):
            yield string

    if omit is False:
        yield '</%s>' % name

class default(object):
    __slots__ = 'value',

    def __init__(self):
        self.value = None
        
    def __setitem__(self, key, value):
        self.value = value

    def __html__(self):
        return self.value
        
class econtext(dict):
    """Dynamic scope dictionary which is compatible with the
    `econtext` of ZPT."""
    
    set_local = setLocal = dict.__setitem__
    set_global = setGlobal = dict.__setitem__

    @property
    def vars(self):
        return self
    
class scope(list):
    def __init__(self, *args):
        global s_counter
        self.hash = s_counter
        s_counter += 1

    def __hash__(self):
        return self.hash

class emptydict(dict):
    def __setitem__(self, key, value):
        raise TypeError("Read-only dictionary does not support assignment.")
    
class repeatitem(object):
    interface.implements(interfaces.ITALESIterator)
    
    def __init__(self, iterator, length):
        self.length = length
        self.iterator = iterator
        
    @property
    def index(self):
        try:
            length = len(self.iterator)
        except TypeError:
            length = self.iterator.__length_hint__()
        except:
            raise TypeError("Unable to determine length.")

        try:
            return self.length - length - 1
        except TypeError:
            return None
            
    @property
    def start(self):
        return self.index == 0

    @property
    def end(self):
        return self.index == self.length - 1

    def number(self):
        return self.index + 1

    def odd(self):
        return bool(self.index % 2)

    def even(self):
        return not self.odd()

class repeatdict(dict):
    def insert(self, key, iterable):
        """We realize the iterable and extract a new iterable from it;
        this allows us to keep track of the iteration and make
        available the repeat dictionary."""

        if not isinstance(iterable, (tuple, list)):
            try:
                if iterable is None:
                    iterable = ()
                else:
                    iterable = tuple(iterable)
            except TypeError:
                # The message below to the TypeError is the Python
                # 2.5-style exception message. Python 2.4.X also
                # raises a TypeError, but with a different message.
                # ("TypeError: iteration over non-sequence").  The
                # Python 2.5 error message is more helpful.  We
                # construct the 2.5-style message explicitly here so
                # that both Python 2.4.X and Python 2.5+ will raise
                # the same error.  This makes writing the tests eaiser
                # and makes the output easier to understand.
                raise TypeError("%r object is not iterable" %
                                type(iterable).__name__)
            
        iterator = iter(iterable)
        length = len(iterable)
        self[key] = (iterator, length)
        
        return iterator
        
    def __getitem__(self, key):
        value, length = dict.__getitem__(self, key)

        if not isinstance(value, repeatitem):
            value = repeatitem(value, length)
            self[key] = (value, length)
            
        return value

    def get(self, key, default):
        if key not in self:
            return default
        return self[key]
        
class odict(UserDict):
    def __init__(self, dict = None):
        self._keys = []
        UserDict.__init__(self, dict)

    def __delitem__(self, key):
        UserDict.__delitem__(self, key)
        self._keys.remove(key)

    def __setitem__(self, key, item):
        """Case insensitive set item."""
        
        keys = tuple(key.lower() for key in self._keys)
        _key = key.lower()
        if _key in keys:
            for k in self._keys:
                if k == _key:
                    key = k
                    break
                elif k.lower() == _key:
                    self._keys.remove(k)
                    key = k
                    break

        UserDict.__setitem__(self, key, item)
        
        if key not in self._keys:
            self._keys.append(key)

    def clear(self):
        UserDict.clear(self)
        self._keys = []

    def copy(self):
        dict = UserDict.copy(self)
        dict._keys = self._keys[:]
        return dict

    def items(self):
        return zip(self._keys, self.values())

    def keys(self):
        return self._keys

    def popitem(self):
        try:
            key = self._keys[-1]
        except IndexError:
            raise KeyError('dictionary is empty')

        val = self[key]
        del self[key]

        return (key, val)

    def setdefault(self, key, failobj = None):
        UserDict.setdefault(self, key, failobj)
        if key not in self._keys: self._keys.append(key)

    def update(self, dict):
        UserDict.update(self, dict)
        for key in dict.keys():
            if key not in self._keys: self._keys.append(key)

    def values(self):
        return map(self.get, self._keys)

def get_attributes_from_namespace(element, namespace):
    if namespace is None:
        attrs = dict([
            (name, value) for (name, value) in element.attrib.items() \
            if '{' not in name])
    else:
        attrs = dict([
            (name, value) for (name, value) in element.attrib.items() \
            if name.startswith('{%s}' % namespace)])

    return attrs

def format_attribute(element, key):
    if '}' not in key:
        return key
    ns, name = key[1:].split('}')
    for prefix, namespace in element.nsmap.items():
        if ns == namespace:
            return '%s:%s' % (prefix, name)
    for prefix, namespace in config.DEFAULT_NS_MAP.items():
        if ns == namespace:
            return '%s:%s' % (prefix, name)
    raise ValueError("Can't format attribute: %s." % key)

def get_namespace(element):
    if '}' in element.tag:
        return element.tag.split('}')[0][1:]
    return element.nsmap.get(None)

def format_kwargs(kwargs):
    items = []
    for name, value in kwargs.items():
        if isinstance(value, (int, float, basestring)):
            items.append((name, value))
        elif isinstance(value, dict):
            items.append((name, '{...} (%d)' % len(value)))
        else:
            items.append((name,
                "<%s %s at %s>" % (
                    type(value).__name__,
                    getattr(value, '__name__', "-"),
                    hex(id(value)))))
    
    return ["%s: %s" % item for item in items]

def raise_exc(exc):
    raise exc

def raise_template_exception(source, description, kwargs, exc_info):
    """Re-raise exception raised while calling ``template``, given by
    the ``exc_info`` tuple (see ``sys.exc_info``)."""

    kwargs = kwargs.copy()
    
    # merge dynamic scope
    kwargs.update(kwargs.pop('econtext', {}))

    # omit keyword arguments that begin with an underscore; these are
    # used internally be the template engine and should not be exposed
    map(kwargs.__delitem__, [k for k in kwargs if k.startswith('_')])
    
    # format keyword arguments; consecutive arguments are indented for
    # readability
    try:
        formatted_arguments = format_kwargs(kwargs)
    except:
        # the ``pprint.pformat`` method calls the representation
        # method of the arguments; this may fail and since we're
        # already in an exception handler, there's no point in
        # pursuing this further
        formatted_arguments = ()
    
    for index, string in enumerate(formatted_arguments[1:]):
        formatted_arguments[index+1] = " "*15 + string

    # extract line number from traceback object
    cls, exc, tb = exc_info
    try:
        lineno = tb.tb_next.tb_lineno-1
    except AttributeError:
        lineno = 1
        
    # locate source code annotation (these are available from
    # the template source as comments)
    if source:
        source = source.split('\n')
        for i in reversed(range(lineno)):
            m = re_annotation.match(source[i])
            if m is not None:
                annotation = m.group(1)
                break
        else:
            annotation = ""

        error_msg = "Caught exception rendering template."
    else:
        annotation = ""
        error_msg = "Caught exception processing template: %s.\n" % description

    __dict__ = exc.__dict__
    __name__ = cls.__name__
    
    error_string = str(exc)

    if issubclass(cls, Exception):
        class RuntimeError(cls):
            def __init__(self, *args, **kwargs):
                pass
            
            def __str__(self):
                return "%s\n%s\n%s: %s" % (
                    error_msg, str(self.__debug_info__),
                    __name__, error_string)

        if isinstance(cls, types.TypeType):
            exc = RuntimeError.__new__(RuntimeError)
            exc.__dict__.update(__dict__)
        else:
            cls = RuntimeError

        # add/extend debug info
        try:
            debug_info = RuntimeError.__debug_info__
        except AttributeError:
            debug_info = RuntimeError.__debug_info__ = DebugInfo()
        debug_info.append(annotation, description, formatted_arguments)

    raise cls, exc, tb

class DebugInfo(object):
    def __init__(self):
        self.info = []
        
    def append(self, *args):
        self.info.append(args)

    def __str__(self):
        error_format = ' - Expression: "%s"\n' \
                       ' - Instance:   %s\n' \
                       ' - Arguments:  %s\n'

        annotation, description, formatted_arguments = self.info[-1]
        return error_format % (
            annotation, description, "\n".join(formatted_arguments))

# importing these here to avoid circular reference

import config

def xhtml_attr(name):
    return "{%s}%s" % (config.XHTML_NS, name)

def tal_attr(name):
    return "{%s}%s" % (config.TAL_NS, name)

def meta_attr(name):
    return "{%s}%s" % (config.META_NS, name)

def metal_attr(name):
    return "{%s}%s" % (config.METAL_NS, name)

def i18n_attr(name):
    return "{%s}%s" % (config.I18N_NS, name)

def py_attr(name):
    return "{%s}%s" % (config.PY_NS, name)
